/*
 * Copyright (c) 1993 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the Computer Systems
 *	Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @(#) $Header: command.h,v 1.8 95/07/30 14:26:25 mccanne Exp $ (LBL)
 */

#ifndef ns_command_h
#define ns_command_h

#include "Tcl.h"

class CommandHandler {
 public:
	CommandHandler(const char* name);
	virtual int command(int argc, const char*const* argv) = 0;
	inline const char* name() const { return (name_); }
 protected:
	CommandHandler* next_;
	const char* name_;
};

class Var;
class Trace;

class Object {
 public:
	Object(int prefixChar);
	virtual ~Object();
	virtual int command(int argc, const char*const* argv);
	virtual void reset();
	inline const char* proc() const { return (proc_); }
	void link_bool(const char* a, const char* fld, int* val, int readOnly);
	void link_int(const char* a, const char* fld, int* val, int readOnly);
	void link_real(const char* a, const char* fld,
		       double* val, int readOnly);
	void link_bw(const char* a, const char* fld,
		     double* val, int readOnly);
	void link_time(const char* a, const char* fld,
		     double* val, int readOnly);
	inline int id() const { return (id_); }
 protected:
	void insert(Var* p);
	Var* varlookup(const char* var) const;
	int get(const char* var);
	int set(const char* val, const char* val);
	void linkvar(const char* fld, char* val, int flag, int readOnly);
	static int command(ClientData, Tcl_Interp*, int argc, char **argv);
	static int nid_;
	int id_;
	char proc_[16];
	Var* var_;
	Trace* trace_;
 private:
	Object* next_;
	static Object* all_;
 public:
	static Object* lookup(const char* name);
	static void reset_all();
	static void list(int prefix, char* cp);

	static Object* first(int prefix) {
		return (next(prefix, all_));
	}
	static Object* next(int prefix, Object* o) {
		for (o = o->next_; o != 0; o = o->next_)
			if (o->proc_[1] == prefix)
				break;
		return (o);
	}
};

#define CMD_HASH_SIZE 64

class CommandTable {
 public:
	static int command(ClientData, Tcl_Interp*, int argc, char **argv);
	static void enter(CommandHandler*);
 protected:
	static int hash(const char*);
	static CommandHandler* hashtab_[CMD_HASH_SIZE];
};
#endif
