/*
 * Copyright (c) 1993 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by the Computer Systems
 *	Engineering Group at Lawrence Berkeley Laboratory.
 * 4. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @(#) $Header: node.h,v 1.3 95/07/30 14:29:00 mccanne Exp $ (LBL)
 */

#ifndef ns_node_h
#define ns_node_h

#include "command.h"
#include "scheduler.h"

class Link;
class Packet;
class Agent;

#define MRHSIZE 256
#define MRHASH(s, g) ((s ^ g) & (MRHSIZE-1))

struct mlink  {
	mlink* next;
	int prune;
	Link* link;
};

struct mrt {
	int src;
	int grp;
	mlink* links;
	Agent* agents;
	mrt* next;
};

class Node : public Object, public Handler {
 public:
	Node();
	~Node();
	void handle(Event*);
	inline void transmit(Packet* p) { handle((Event*)p); }
	static Node* lookup(const char* name) {
		return ((Node*)Object::lookup(name));
	}
	void add_link(Link*);	/* add an outgoing link to this node */
	void setroute(int dst, Link*);
	Link* links() const { return (links_); }
	int add_agent(Agent*);
	void join_group(Agent*, int group);
	void leave_group(Agent*, int group);
	inline int addr() const { return (addr_); }
	static inline int cnt() { return (cnt_); }
 protected:
	virtual int command(int argc, const char*const* argv);
	struct AgentList {
		AgentList* next;
		Agent* agent;
	};
	void free(AgentList*);
	static int cnt_;	/* total number of nodes */
	int addr_;		/* address */
	int nport_;		/* number of local agents (each has a port) */
	Link** route_;		/* routing table */
	Link* links_;		/* list of outgoing links */
	int maxport_;
	Agent** demux_;		/* table of local agents */
	int ngroup_;
	AgentList** group_;	/* table of local multicast listeners */
	mrt* mrtab_[MRHSIZE];		/* multicast routing table */
	mrt* mlookup(int src, int grp);
	void compute_mroute(mrt*);
};
#endif
